﻿using System;
using System.Linq;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using SimpleJSON;
using MacGruber;

namespace SPQR
{
    public class SQPRDynamicText : MVRScript
    {

        private EventTrigger myEventTrigger;

        protected UIDynamicButton dbuttonReset;
        protected UIDynamicButton dbuttonReplace;

        public JSONStorableString myString;
        public JSONStorableString searchString;
        public JSONStorableString replaceString;
        public JSONStorableString textOriginal;

        JSONStorableString[] customSearch;
        JSONStorableString[] customReplace;
        int customCount = 20;
        int titles = 0;



        public override void Init()
        {
            if(containingAtom.type != "UIText") return;

            try
            {
                Utils.OnInitUI(CreateUIElement);

                CreateTitle("DYNAMIC TEXT",50,false);
                CreateTitle("AETERNUM TOOLS",30,true);
                
                


                JSONStorableString jstring = new JSONStorableString("instructions2", "\n The replace fields can be either direct text or references to other Text atoms in the scene. References must start with '@'. Multiple values can pe given separated by '|' and one will be picked randomly. Works recursively to a max depth of 10.");
                UIDynamicTextField dtext = CreateTextField(jstring, true);
                dtext.UItext.supportRichText = true;
                dtext.backgroundColor = new Color(0.42f, 0.03f, 0.51f, 0.7f);
                dtext.textColor = new Color(1f, 1f, 1f, 0.9f);
                dtext.height = 350f;

                JSONStorableString jstring2 = new JSONStorableString("instructions1", "\n The plugin needs to be attached to a text atom and will enable rich-text for it. Google 'unity rich text' for syntax. The Dynamic Text holder field holds the original text (OriginalText trigger). DynamicTextReplaceVariables replaces all the custom variables in it and updates the text atom.");
                UIDynamicTextField dtext2 = CreateTextField(jstring2);
                dtext2.UItext.supportRichText = true;
                dtext2.backgroundColor = new Color(0.42f, 0.03f, 0.51f, 0.7f);
                dtext2.textColor = new Color(1f, 1f, 1f, 0.9f);
                dtext2.height = 350f;
                

                Utils.SetupSpacer(this, 70, false);
                Utils.SetupSpacer(this, 70, true);


                CreateTitle("1. Dynamic text holder",35,false);
                CreateTitle("Add here the original text that will have variables inside it replaced.",25,true);

                
                textOriginal = new JSONStorableString("OriginalText", "", UpdateMyOriginalString);
                RegisterString(textOriginal);
                UIDynamicLabelInputTextarea textOriginalInput = Utils.SetupTextInputTextarea(this, "Text:", textOriginal, false);

                textOriginalInput.input.onValueChanged.AddListener(textOriginalInputChanged);
                textOriginalInput.input.textComponent.supportRichText = false;

                Utils.SetupSpacer(this, 70, false);
                Utils.SetupSpacer(this, 145, true);

                CreateTitle("2. Text Atoms Variables",35,false);
                CreateTitle("Left: the string to look for. Right: the new value or the atom id.",25,true);

                customSearch = new JSONStorableString[customCount];
                customReplace = new JSONStorableString[customCount];
                for (var i = 0; i < customCount; i++)
                {
                    customSearch[i] = new JSONStorableString("customSearch" + (i + 1), "", UpdateMyString);
                    RegisterString(customSearch[i]);
                    Utils.SetupTextInput(this, "Variable " + (i + 1), customSearch[i], false);

                    customReplace[i] = new JSONStorableString("customReplace" + (i + 1), "", UpdateMyString);
                    RegisterString(customReplace[i]);
                    Utils.SetupTextInput(this, "Replace " + (i + 1), customReplace[i], true);
                }
                
                dbuttonReset = CreateButton("DynamicTextReplaceVariables",true);
                if (dbuttonReset != null)
                {
                    dbuttonReset.button.onClick.AddListener(ResetDynamicText);
                }
                Utils.SetupAction(this, "DynamicTextReplaceVariables", ResetDynamicText);


                Utils.SetupSpacer(this, 115, false);
                Utils.SetupSpacer(this, 50, true);

                CreateTitle("Advanced ",35,false);
                CreateTitle("Use these fields and action to programatically replace the text.",25,true);

                searchString = new JSONStorableString("DynamicTextSearchFor", "", UpdateMyString);
                RegisterString(searchString);
                UIDynamicLabelInput searchStringInput = Utils.SetupTextInput(this, "Search", searchString, false);

                replaceString = new JSONStorableString("DynamicTextReplaceWith", "", UpdateMyString);
                RegisterString(replaceString);
                UIDynamicLabelInput replaceStringInput = Utils.SetupTextInput(this, "Replace:", replaceString, true);



                dbuttonReplace = CreateButton("DynamicTextReplaceSingle", true);
                if (dbuttonReplace != null)
                {
                    dbuttonReplace.button.onClick.AddListener(ReplaceDynamicText);
                }
                Utils.SetupAction(this, "DynamicTextReplaceSingle", ReplaceDynamicText);

                Utils.SetupSpacer(this, 50, false);
                Utils.SetupSpacer(this, 50, true);




                InitEnableRichText();

                SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;
                UIUpdate();
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }
        private void OnAtomRename(string oldid, string newid)
        {
            for (var i = 0; i < customCount; i++)
            {
                if(oldid == customReplace[i].val){
                    customReplace[i].val = newid;
                }
            }

        }
        private void UIUpdate()
        {
        }



        void Start()
        {
            try
            {
                InitSetText();

            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        void Update()
        {
            try
            {
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }


        void FixedUpdate()
        {
            try
            {
                // put code in here
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        void OnDestroy()
        {
            SuperController.singleton.onAtomUIDRenameHandlers -= OnAtomRename;
            Utils.OnDestroyUI();
        }






        //////////////////////////////////////////////
        private void CreateTitle(string text, int fontsize, bool right,int spacing){
            CreateTitle(text, fontsize, right);
            Utils.SetupSpacer(this, spacing, right);
        }
        private void CreateTitle(string text, int fontsize, bool right){
                JSONStorableString title1 = new JSONStorableString("title"+(++titles), "\n<size="+fontsize+"><b>"+text+"</b></size>");
                UIDynamicTextField title1text = CreateTextField(title1, right);
                title1text.UItext.supportRichText = true;
                title1text.backgroundColor = new Color(0f, 0f, 0f, 0f);
                title1text.textColor = new Color(0.42f, 0.03f, 0.51f, 1f);
                title1text.height = 100f;
                title1text.UItext.alignment = right ? TextAnchor.MiddleRight : TextAnchor.MiddleLeft;
        }

        void textOriginalInputChanged(string t)
        {   
            InitSetText(t);
        }

        void InitEnableRichText()
        {
            GetAtomText().supportRichText = true;
        }


        void InitSetText()
        {
            SetText(textOriginal.val);
        }
        void InitSetText(string t)
        {
            SetText(t);
        }

        public String GetText()
        {
            JSONStorable f = containingAtom.GetStorableByID("Text");
            JSONStorableString fs = f.GetStringJSONParam("text");
            return fs.val;
        }


        void SetText(string t)
        {

            for(var i=0; i<customCount; i++){
                if(customSearch[i].val.Length > 0 && customReplace[i].val.Length > 0 ){
                    String replaceWith = GetAtomTextValue(customReplace[i].val);
                    if(replaceWith == null) continue;
                    if(containingAtom.name == customReplace[i].val) continue;
                    t = t.Replace(customSearch[i].val, replaceWith);
                }
            }
            //inception
            for(var i=0; i<customCount; i++){
                if(customSearch[i].val.Length > 0 && customReplace[i].val.Length > 0 ){
                    String replaceWith = GetAtomTextValue(customReplace[i].val);
                    if(replaceWith == null) continue;
                    if(containingAtom.name == customReplace[i].val) continue;
                  
                    t = t.Replace(customSearch[i].val, replaceWith);
                }
            }

            SetAtomText(t);
        }

         void SetAtomText(string t){ 
            JSONStorable f = containingAtom.GetStorableByID("Text");
            JSONStorableString fs = f.GetStringJSONParam("text");
            fs.val = t;
         }

        Text GetAtomText()
        {
            return GetAtomText(containingAtom);
        }

        Text GetAtomText(Atom a)
        {
            TextStorable t = a.GetStorableByID("Text") as TextStorable;
            if (t == null) return null;
            return t.displayField;
        }

        Text GetAtomText(string id)
        {
            Atom a = SuperController.singleton.GetAtomByUid(id);
            if(a == null) return null;
            return GetAtomText(a);
        }


        

        String GetAtomTextValue()
        {
            return GetAtomTextValue(containingAtom);
        }


        String GetAtomTextValue(Atom a)
        {
            return GetAtomTextValue(a,0);
        }
        String GetAtomTextValue(Atom a,int depth)
        {
            JSONStorable f = a.GetStorableByID("Text");
            JSONStorableString fs = f.GetStringJSONParam("text");
            return GetAtomTextValue(fs.val,depth);
        }
        String GetAtomTextValue(string sid){
            return  GetAtomTextValue(sid,0);
        }
        String GetAtomTextValue(string sid, int depth)
        {
            if(depth++ > 10) return sid;
            string[] pairs = sid.Split('|');
            if(pairs.Count() < 1) return "";
            var picked = pairs[(int)UnityEngine.Random.Range(0, pairs.Count())];
            return GetAtomTextValueAtom(picked, depth);
        }

        String GetAtomTextValueAtom(string sid, int depth)
        {   if(sid==null || sid =="") return"";
            if(sid[0]=='@'){ //atom
                string id = sid.Remove(0, 1);
                Atom a = SuperController.singleton.GetAtomByUid(id);
                if(a == null) return null;
                return GetAtomTextValue(a,depth);
            } 

            for(var i=0; i<customCount; i++){
                if(customSearch[i].val.Length > 0  && customReplace[i].val.Length > 0 && customSearch[i].val == sid){
                    return GetAtomTextValue(customReplace[i].val,depth);
                }
            }

            return sid;
        }

        void UpdateMyOriginalString(string name)
        {
            InitSetText();
        }


        void UpdateMyString(string name)
        {
        }

        void ResetDynamicText()
        {
            InitSetText();
        }

        void ReplaceDynamicText()
        {
            string searchFor = searchString.val;
            string replaceWith = replaceString.val;
             string currentText = GetAtomTextValue();
            // string currentText = textOriginal.val;
            if(searchFor.Length > 0){
                currentText = currentText.Replace(searchFor, replaceWith);
            }
            SetText(currentText);


        }
         

    }




}