﻿using System;
using UnityEngine;
using System.Text;

using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using MacGruber;
using Random = UnityEngine.Random;

namespace SPQR
{
    public class SPQRDynamicTriggers : MVRScript {

        private EventTrigger previousEventTrigger;
        private EventTrigger myEventTrigger;

        int titles = 0;
        protected JSONStorableString myString;
    
        public override void Init()
        {
             if(containingAtom.type != "UIText") return;
            Utils.OnInitUI(CreateUIElement);


            
            CreateTitle("DYNAMIC Triggers",50,false);
            CreateTitle("AETERNUM TOOLS",30,true);
            
            


            JSONStorableString jstring = new JSONStorableString("instructions2", "\n This plugin allows settings up triggers with dynamic variables inside them like {i}. You can then set {i} as the value to be replaced in the Search For field. When the plugin trigger gets called, it will replaced all the {i} values it founds in any references with whatever is in the containing text atom at that time. ");
            UIDynamicTextField dtext = CreateTextField(jstring, true);
            dtext.UItext.supportRichText = true;
            dtext.backgroundColor = new Color(0.42f, 0.03f, 0.51f, 0.7f);
            dtext.textColor = new Color(1f, 1f, 1f, 0.9f);
            dtext.height = 350f;

            JSONStorableString jstring2 = new JSONStorableString("instructions1", "\n Needs to be attached to a Text atom. The trigger can be called from vam buttons and other triggers by calling  atomName > pluginName > DynamicTrigger. An example in the demo scene is in the varCurrentState atom on the first plugin on its OnUpdate trigger.");
            UIDynamicTextField dtext2 = CreateTextField(jstring2);
            dtext2.UItext.supportRichText = true;
            dtext2.backgroundColor = new Color(0.42f, 0.03f, 0.51f, 0.7f);
            dtext2.textColor = new Color(1f, 1f, 1f, 0.9f);
            dtext2.height = 350f;
            

            Utils.SetupSpacer(this, 70, false);
            Utils.SetupSpacer(this, 70, true);




            myEventTrigger = new EventTrigger(this, "OnTrigger");
            Utils.SetupButton(this, "Actions OnTrigger", myEventTrigger.OpenPanel, false);

            Utils.SetupButton(this, "Test Trigger", TestTrigger, true);
            Utils.SetupAction(this, "DynamicTrigger", TestTrigger);
            SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;
            SimpleTriggerHandler.LoadAssets();


            Utils.SetupSpacer(this, 50, false);
            Utils.SetupSpacer(this, 50, true);

            myString = new JSONStorableString("SearchFor", "", UpdateMyString);
            RegisterString(myString);
            UIDynamicLabelInput myStringInput = Utils.SetupTextInput(this, "Search For", myString, false);

            SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;
            UIUpdate();
        }
        private void OnAtomRename(string oldid, string newid)
        {
            myEventTrigger.SyncAtomNames();
        }
        private void UIUpdate()
        {
        }


        void Start()
        {
            try
            {
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        void Update()
        {
             if(containingAtom.type != "UIText") return;
            try
            {
                myEventTrigger.Update();
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }
        private void TestTrigger()
        {
            UpdateEvent();

        }
        private void Trigger()
        {
            myEventTrigger.Trigger();
            
        }

        void FixedUpdate()
        {
            try
            {
                // put code in here
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        void OnDestroy()
        {
            SuperController.singleton.onAtomUIDRenameHandlers -= OnAtomRename;
            myEventTrigger.Remove();
            Utils.OnDestroyUI();
        }



        public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
        {
            JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
            if (includePhysical || forceStore)
            {
                needsStore = true;
                jc[myEventTrigger.Name] = myEventTrigger.GetJSON(base.subScenePrefix);
            }
            return jc;
        }

        public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
        {
            base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);

            if (!base.physicalLocked && restorePhysical && !IsCustomPhysicalParamLocked("trigger"))
            {
                myEventTrigger.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
            }
        }



        //////////////////////////////////////////////

        void UpdateMyString(string name)
        {
        }


        void UpdateEvent(){
            string rawJSON = this.GetJSON(true, true, false).ToString();
            string originalRawJson = rawJSON;
            rawJSON = rawJSON.Replace(myString.val,GetText());
            RestoreTrigger(rawJSON);
            Trigger();
            RestoreTrigger(originalRawJson);
            
        }

        void RestoreTrigger(string raw){
            var jc = this.GetJSON(true,true,false);
            JSONClass njc = JSON.Parse(raw) as JSONClass;
            if(njc["id"] == null || njc["OnTrigger"] == null) return;
            jc["OnTrigger"] = njc["OnTrigger"];
            LateRestoreFromJSON(jc);
            
        }

          public string GetText()
        {
            JSONStorable f = containingAtom.GetStorableByID("Text");
            JSONStorableString fs = f.GetStringJSONParam("text");
            return fs.val;
        }

        private void CreateTitle(string text, int fontsize, bool right,int spacing){
            CreateTitle(text, fontsize, right);
            Utils.SetupSpacer(this, spacing, right);
        }
        private void CreateTitle(string text, int fontsize, bool right){
                JSONStorableString title1 = new JSONStorableString("title"+(++titles), "\n<size="+fontsize+"><b>"+text+"</b></size>");
                UIDynamicTextField title1text = CreateTextField(title1, right);
                title1text.UItext.supportRichText = true;
                title1text.backgroundColor = new Color(0f, 0f, 0f, 0f);
                title1text.textColor = new Color(0.42f, 0.03f, 0.51f, 1f);
                title1text.height = 100f;
                title1text.UItext.alignment = right ? TextAnchor.MiddleRight : TextAnchor.MiddleLeft;
        }



	}
}